

var target : Transform;

private var throttle = 0.0;
private var dist2targ : float;

// The camera doesnt start following the target immediately but waits for a split second to avoid too much waving around.
private var lockCameraTimer = 0.0;

// The current move direction in x-z
private var moveDirection = Vector3.zero;
private var forward = Vector3.zero;
private var right = Vector3.zero;
// The current vertical speed
private var verticalSpeed = 0.0;
// The current x-z move speed
private var moveSpeed = 0.0;

// The last collision flags returned from controller.Move
private var collisionFlags : CollisionFlags; 

private var isControllable = true;

private var puppet : ShipModel;
private var current_weapon : Weapon;

function Awake ()
{
	moveDirection = transform.TransformDirection(Vector3.forward);
    forward = transform.TransformDirection(Vector3.forward);
	right = Vector3(forward.z, 0, -forward.x);
}

function Start()
{
	puppet = GetComponent(ShipModel);

    current_weapon = transform.Find("Weapon").GetComponent(Weapon);
}

function SetTarget(newtarget) {
    target = newtarget;
}

function findTarget() {
    var nearest_dist = 999999999999.9;
    var nearest_ship : Transform = null;
    for(var ship : GameObject in GameObject.FindGameObjectsWithTag("Enemy")) {
        var this_team = ship.GetComponent(ShipModel).team;
        if (this_team != puppet.team) {
            var this_dist = Vector3.Distance(transform.position, ship.transform.position);
            if (this_dist < nearest_dist) {
                nearest_dist = this_dist;
                nearest_ship = ship.transform;
            }
        }
    }

    return nearest_ship;
}

// This next function responds to the "HidePlayer" message by hiding the player. 
// The message is also 'replied to' by identically-named functions in the collision-handling scripts.
// - Used by the LevelStatus script when the level completed animation is triggered.

function HidePlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = false; // stop rendering the player.
	isControllable = false;	// disable player controls.
}

// This is a complementary function to the above. We don't use it in the tutorial, but it's included for
// the sake of completeness. (I like orthogonal APIs; so sue me!)

function ShowPlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = true; // start rendering the player again.
	isControllable = true;	// allow player to control the character again.
}



function Fire() {
    var aim_point = target.position;
    //var aim_point = transform.position + transform.TransformDirection(Vector3.forward) * 100.0;
    BroadcastMessage("Pewpew", aim_point, SendMessageOptions.DontRequireReceiver);
}


function turnTo(direction : Vector3) {
    var destination_rot = Quaternion.LookRotation(direction);
    //var difference = destination_rot - transform.rotation;

    var str = Mathf.Min (puppet.yaw * Time.deltaTime, 1); 

    transform.rotation = Quaternion.Slerp(transform.rotation, destination_rot, str);
    //RotateTowards(transform.rotation, destination_rot, str, 1.0);
}

function seek(destination : Vector3) {
    turnTo(destination - transform.position);
}
function avoid(destination : Vector3) {
    turnTo(transform.position - destination);
}



function isInsideArc(aim_point : Vector3) {
    var arc = current_weapon.firing_arc;
    forward = transform.TransformDirection(Vector3.forward);
    var angleto = Vector3.Angle(forward, aim_point);
    //Debug.Log("angleto is " + angleto);
    if (angleto <= arc) { return true; }
    else { return false; }
}



function Update() {
    forward = transform.TransformDirection(Vector3.forward);

    if( target ) {
        seek(target.position);
        throttle = 1.0;
        puppet.Thrust(throttle);

            // grab the firing arc of one of my armed weapon
        if (isInsideArc(target.position)) {
            dist2targ = Vector3.Distance(puppet.transform.position, target.position);
            if( dist2targ < 5.0 ) {
                Fire();
            }
            /*
            var difference = target.position - transform.position;
            aim_dot = Vector3.Dot(forward, difference.Normalize());

            if( aim_dot < 1.0 - firing_arc) {

            }
            */
            //puppet.SetAim(Ray(puppet.transform.position, target.position));
            //BroadcastMessage("Aim_with_slerp", target.position, SendMessageOptions.DontRequireReceiver);
        }
    }
    else {
        target = findTarget();
    }
	
	if (!isControllable)
	{
		// kill all inputs if not controllable.
		Input.ResetInputAxes();
	}

    //FixedUpdate();
}


function GetSpeed () {
	return moveSpeed;
}

function GetThrottle() {
    return throttle;
}

function GetDirection () {
	return moveDirection;
}


function GetLockCameraTimer () 
{
	return lockCameraTimer;
}

function IsMoving ()  : boolean
{
	return Mathf.Abs(Input.GetAxisRaw("Vertical")) + Mathf.Abs(Input.GetAxisRaw("Horizontal")) > 0.5;
}


@script RequireComponent(ShipModel)
@script AddComponentMenu("Oolong/Ship (Controller -- AI)")
