
// The speed when walking
var impulse = 10.0;

var yaw = 25.0;
var pitch = 25.0;
var roll = 35.0;

var deadzone = 0.15;

var projectile : Rigidbody;

private var throttle = 0.0;
private var cursor = Vector3.zero;

// The camera doesnt start following the target immediately but waits for a split second to avoid too much waving around.
private var lockCameraTimer = 0.0;

// The current move direction in x-z
private var moveDirection = Vector3.zero;
private var forward = Vector3.zero;
private var right = Vector3.zero;
// The current vertical speed
private var verticalSpeed = 0.0;
// The current x-z move speed
private var moveSpeed = 0.0;

// The last collision flags returned from controller.Move
private var collisionFlags : CollisionFlags; 

// Are we moving backwards (This locks the camera to not do a 180 degree spin)
private var movingBack = false;
// Is the user pressing any keys?
private var isMoving = false;
// When did the user start walking (Used for going into trot after a while)
private var walkTimeStart = 0.0;


private var isControllable = true;

function Awake ()
{
	moveDirection = transform.TransformDirection(Vector3.forward);
    forward = transform.TransformDirection(Vector3.forward);
	right = Vector3(forward.z, 0, -forward.x);
}

// This next function responds to the "HidePlayer" message by hiding the player. 
// The message is also 'replied to' by identically-named functions in the collision-handling scripts.
// - Used by the LevelStatus script when the level completed animation is triggered.

function HidePlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = false; // stop rendering the player.
	isControllable = false;	// disable player controls.
}

// This is a complementary function to the above. We don't use it in the tutorial, but it's included for
// the sake of completeness. (I like orthogonal APIs; so sue me!)

function ShowPlayer()
{
	GameObject.Find("rootJoint").GetComponent(SkinnedMeshRenderer).enabled = true; // start rendering the player again.
	isControllable = true;	// allow player to control the character again.
}



function Thrust(scale) {    
    // get the body
    rigidbody.AddForce(forward * scale * impulse * Time.fixedDeltaTime);
}


function Turn(direction) {
    var xrot = pitch * direction.x * Time.fixedDeltaTime;
    var yrot = yaw * direction.y * Time.fixedDeltaTime;
    var zrot = roll * direction.z * Time.fixedDeltaTime;
    var xQuaternion = Quaternion.AngleAxis (xrot, Vector3.right); 
    var yQuaternion = Quaternion.AngleAxis (yrot, Vector3.up); 
    var zQuaternion = Quaternion.AngleAxis (zrot, Vector3.forward); 

    //var deltaRotation = Quaternion.Euler(xQuaternion);
    var totalRotation = xQuaternion * yQuaternion * zQuaternion;
    //var totalRotation = Vector3(direction.x * pitch, direction.y * yaw, direction.z * roll);
    //totalRotation *= Time.fixedDeltaTime;

    rigidbody.MoveRotation(rigidbody.rotation * totalRotation);
    //Debug.Log("Rotation is: " + rigidbody.rotation);
    //transform.Rotate(totalRotation);
    //rigidbody.rotation = transform.rotation;
}


function clamp(value, min, max) {
    if(value < min){ value = min; }
    else if(value > max){ value = max; }
    return value;
}

function FixedUpdate() {
	if (Input.GetButton ("Accelerate"))
	{
        var delta = Input.GetAxis("Accelerate");
        if(throttle < 1.0 && delta > 0.0) {
          throttle += Time.fixedDeltaTime * delta;
        }
        else if(throttle > 0.0 && delta < 0.0) {
          throttle += Time.fixedDeltaTime * delta;
        }
        throttle = clamp(throttle, 0.0, 1.0);
	}
    Thrust(throttle);

    cursor.x = (Input.mousePosition.x - Screen.width/2) / (Screen.width/2);
    cursor.y = (Input.mousePosition.y - Screen.height/2) / (Screen.height/2);

    // clamp cursor
    cursor.x = clamp(cursor.x, -1.0, 1.0);
    cursor.y = clamp(cursor.y, -1.0, 1.0);

    var turnx = 0.0;
    var turny = 0.0;

    // clamp the cursor to the deadzone
    if(cursor.x > deadzone) {
        turnx = (cursor.x - deadzone) / (1.0 - deadzone);
    }
    else if (cursor.x < -deadzone) {
        turnx = (cursor.x + deadzone) / (1.0 - deadzone);
    }
    else {
        turnx = 0.0;
    }
    if(cursor.y > deadzone) {
        turny = (cursor.y - deadzone) / (1.0 - deadzone);
    }
    else if (cursor.y < -deadzone) {
        turny = (cursor.y + deadzone) / (1.0 - deadzone);
    }
    else {
        turny = 0.0;
    }
    //cursor = Vector3(mousex, mousey, 0);
    var turn_direction = Vector3 (-turny, turnx, Input.GetAxis("Roll"));
    //var turn_direction = Vector3 (Input.GetAxis("Vertical"), Input.GetAxis("Vertical"), Input.GetAxis("Roll"));
    Turn(turn_direction);

    transform.position = rigidbody.position;
    //transform.localRotation = rigidbody.rotation;
}


function Fire() {
    BroadcastMessage("Pewpew", SendMessageOptions.DontRequireReceiver);
}


function Update() {
    forward = transform.TransformDirection(Vector3.forward);
	
	if (!isControllable)
	{
		// kill all inputs if not controllable.
		Input.ResetInputAxes();
	}

    if (Input.GetButton("Fire1")) {
        Fire();
    }
    /*
	// Calculate actual motion
	var movement = moveDirection * moveSpeed + Vector3 (0, verticalSpeed, 0) + inAirVelocity;
	movement *= Time.deltaTime;
	
	// Move the controller
	var controller : CharacterController = GetComponent(CharacterController);
	collisionFlags = controller.Move(movement);
	
	transform.rotation = Quaternion.LookRotation(moveDirection);
    */

    FixedUpdate();
}

function OnControllerColliderHit (hit : ControllerColliderHit )
{
//	Debug.DrawRay(hit.point, hit.normal);
	if (hit.moveDirection.y > 0.01) 
		return;
	wallJumpContactNormal = hit.normal;
}

function GetSpeed () {
	return moveSpeed;
}

function GetThrottle() {
    return throttle;
}

function GetDirection () {
	return moveDirection;
}

function IsMovingBackwards () {
	return movingBack;
}

function GetLockCameraTimer () 
{
	return lockCameraTimer;
}

function IsMoving ()  : boolean
{
	return Mathf.Abs(Input.GetAxisRaw("Vertical")) + Mathf.Abs(Input.GetAxisRaw("Horizontal")) > 0.5;
}


function Reset ()
{
	gameObject.tag = "Player";
}
// Require a character controller to be attached to the same game object
@script AddComponentMenu("Third Person Player/Ship Controller")
